//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef GamePreferencesDialog_H
#define GamePreferencesDialog_H

#include <pgwindow.h>
#include <pgeventobject.h>
#include <pglineedit.h>
#include <pgdropdown.h>

#include "GameScenario.h"
#include "player.h"
#include "CreateScenario.h"

class Player_preferences;
class TerrainConfig;
class CreateScenario;

/* GamePreferencesDialog is the class where the player, or players in a
 * multiplayergame, decide with wich parameters he/they want to play
 */

class GamePreferencesDialog : public PG_EventObject, public PG_ThemeWidget, public SigC::Object
{
	public:
		// CREATORS
		GamePreferencesDialog(PG_Widget* parent, PG_Rect rect);
		~GamePreferencesDialog();

        // MANIPULATORS
        void initGUI();

        // SIGNALS
        SigC::Signal0<void> playerDataChanged;

        // SLOTS
        void slotPlayerDataChanged();

		// ACCESSORS
		int noPlayers();

        bool fillData(CreateScenario* creator);
	
		// CALLBACK FUNCTIONS
		PARAGUI_CALLBACK(randomClicked);
		PARAGUI_CALLBACK(loadClicked);
		PARAGUI_CALLBACK(browseClicked);
		PARAGUI_CALLBACK(okClicked);
        PARAGUI_CALLBACK(sizeClicked);

		// ACCESSORS
		GameScenario::MapType getMapType(){return d_DialogType;}
		std::string getFilename(){return d_fileName;}
		
	private:
		// WIDGETS
        PG_Label* d_l_type;
        PG_Label* d_l_name;
        PG_Label* d_l_armyset;
        PG_Label* d_l_start;
        PG_Label* d_l_tiles;
        Player_preferences* d_player_preferences[8];
        PG_Button* d_b_random;
        PG_Button* d_b_load;
        PG_Button* d_b_browse;
        PG_Button* d_b_ok;
        PG_Button* d_b_normalsize;
        PG_Button* d_b_smallsize;
        PG_Button* d_b_tinysize;
        PG_DropDown* d_d_tiles;
        PG_LineEdit* d_edit;
        TerrainConfig* d_grass;
        TerrainConfig* d_water;
        TerrainConfig* d_swamp;
        TerrainConfig* d_forest;
        TerrainConfig* d_hills;
        TerrainConfig* d_mountains;
        TerrainConfig* d_cities;
        TerrainConfig* d_ruins;
        TerrainConfig* d_temples;

		// DATA
		GameScenario::MapType d_DialogType;
        std::string d_fileName;    // filename of map (if loaded)
};

#endif

// End of file
