//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef __QUEST_ENEMY_ARMIES_H
#define __QUEST_ENEMY_ARMIES_H

    #include "Quest.h"
    #include <sigc++/sigc++.h>
    #include "army.h"
    #include "signals.h"

    /**
     * \brief Quest - kill an amount of enemy armies
     *
     */
    class QuestEnemyArmies : public Quest, public SigC::Object
    {
        public:

            /** \brief Constructor */
            QuestEnemyArmies(QuestsManager& q_mgr, int heroId);

            /** \brief Constructor - create a new quest from
                       saved data */
            QuestEnemyArmies(QuestsManager& q_mgr, int heroId,
                             XML_Helper* helper);
         
            /**
             * \brief Get progress information 
             *
             * \param s here we append the progress information
             */
             void getProgress(std::string& s);

            /**
             * \brief Provide the lines of the message describing
                      the quest completion.
             */
             void getSuccessMsg(std::queue<std::string>& msgs);

             static bool isFeasible();

        private:

            /** slot that would receive the sdyingArmy (signal) */
            void dyingArmy(Army *stack, culpritsType culprits);

            void _initDescription(std::string& d, 
                                  unsigned int how_many_to_kill);

            bool save(XML_Helper* helper);

            /** how many armies we should kill */
            unsigned int d_how_many_to_kill;
            /** how many armies we already have killed */
            unsigned int d_how_many_killed;

    };

#endif
