//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <iostream>
#include <sstream>
#include "army.h"
#include "armyset.h"
#include "counter.h"
#include "GraphicsCache.h"
#include "player.h"

Army::Army(Armyset* armyset, int type, Player* owner)
	:Armyset_army(armyset->army[type]), d_armyset(armyset), d_owner(owner),
    d_type(type), d_hp(getMaxHp()), d_moves(getMaxMoves()),
    d_strength(getMaxStrength()), d_xp(0), d_level(1), d_blessed(false),
    d_grouped(true)
{
    d_id = fl_counter->getNextId();
}

Army::Army(Army& a)
	:Armyset_army(a), d_armyset(a.d_armyset), d_owner(a.d_owner), d_id(a.d_id),
    d_type(a.d_type), d_hp(a.d_hp), d_moves(a.d_moves), 
    d_strength(a.d_strength), d_xp(a.d_xp), d_level(a.d_level),
    d_blessed(a.d_blessed), d_grouped(a.d_grouped)
{
}

Army::Army(Armyset* armyset, int type, XML_Helper* helper)
    :Armyset_army(armyset->army[type]), d_armyset(armyset), d_owner(0),
    d_type(type), d_grouped(true)
{
    //We don't need to load our player. If we have one, it is already assigned
    //by the stack we belong to.
    
    helper->getData(d_id, "id");
    helper->getData(d_hp, "hp");
    helper->getData(d_moves, "moves");
    helper->getData(d_strength, "strength");
    helper->getData(d_xp, "xp");
    helper->getData(d_level, "level");
    helper->getData(d_blessed, "blessed");
}

Army::~Army()
{
}

SDL_Surface* Army::getPixmap()
{
    //use the GraphicsCache to get a picture of the army's armyset_army
    return GraphicsCache::getInstance()->getArmyPic(d_armyset->getName(),
                                                    d_type, d_owner);
}

void Army::resetMoves()
{
	d_moves = getMaxMoves();
}

void Army::bless()
{
	if (!d_blessed)
	{
		d_blessed = true;
		d_strength++;
	}
}

void Army::heal(int hp)
{
    if (hp > 0)
    {
	//add no_hp
	d_hp += hp;
    }
    else
    {
	//healing at end of turn, army has to take care of how much to heal
	//for the beginning, we heal 1/5th of hp's every turn
	// (should be changed somewhen)
	d_hp += (getMaxHp() / 5);
    }

    if (d_hp > getMaxHp())
	d_hp = getMaxHp();
}

bool Army::damage(int damageDone)
{
	d_hp -= damageDone;
    return (d_hp <= 0);
}

void Army::decrementMoves(int moves)
{
	d_moves -= moves;

	//just to make sure
	if (d_moves < 0)
	{
		std::cerr <<"Error: moves of army < 0\n";
		d_moves = 0;
	}
}

void Army::gainXp(int n)
{
	//How do we want to handle experience points?
	d_xp += n;
}

//note: if you change anything in here, don't forget to apply these changes
//to thehero class as well!
bool Army::save(XML_Helper* helper)
{
	bool retval = true;

	retval &= helper->openTag("army");
	retval &= helper->saveData("id", d_id);
	retval &= helper->saveData("type", d_type);
	retval &= helper->saveData("hp", d_hp);
	retval &= helper->saveData("moves", d_moves);
	retval &= helper->saveData("strength", d_strength);
	retval &= helper->saveData("xp", d_xp);
	retval &= helper->saveData("level", d_level);
	retval &= helper->saveData("blessed", d_blessed);
	retval &= helper->closeTag();

	return retval;
}
