//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef CITY_H
#define CITY_H

#include <string>
#include "armyset.h"
#include "Location.h"
#include "xmlhelper.h"

class Player;
class Stack;

typedef struct
{
    std::string armyset;
    int type;
} CityProduction;

class City : public Location
{
    public:
        // CREATORS
        City(PG_Point pos, std::string name = "Noname", int gold = 30);
        City(XML_Helper* helper);
        ~City();

        // MANIPULATORS
        void setPlayer(Player* player){d_player = player;}
        void setGold(int gold){d_gold = gold;}
        void setDefenseLevel(int defense_level){d_defense_level = defense_level;}
        void setProduction(int production);
        void setDuration(int duration){d_duration = duration;}
        void setArmytype(int index, int armytype, Armyset* set = 0);
        void setBurnt(bool burnt){d_burnt = burnt;}
        void setRandomArmytypes();
        void produceArmy();
        void produceLargestArmy();
        void addHero(int heroGender = 0, std::string heroName = "Demonlord");
        void nextTurn();
        bool hasProduction(int type, Armyset* set = 0);
        bool save(XML_Helper* helper);
		
        // ACCESSORS
        unsigned int getId() {return d_id;}
        int getDefenseLevel() const {return d_defense_level;}
        Player* getPlayer() const {return d_player;}
        int getDuration() const {return d_duration;}
        int getProduction() const {return d_production;}
        int getGold() const {return d_gold;}
        int getArmytype(int slot) const {return d_armytype[slot].type;}
        CityProduction getProductiontype(int slot) {return d_armytype[slot];}
        bool isBurnt() const {return d_burnt;}
        bool isFriend(Player* player) const;

    private:
        // MANIPULATORS
        Stack* getFreeStack();

        // DATA
        Player* d_player;		// Owner
		
        int d_duration;			// needed turns to finish current production
        CityProduction d_armytype[4];   // possible productiontypes
        int d_production;		// number of produced armytype
        int d_gold;		    	// gold produced by city each turn
        bool d_burnt;			// is city burnt down?
        int d_defense_level;	// defense of the city
        unsigned int d_id;
};

#endif // CITY_H
