//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <sstream>
#include <pgmessagebox.h>
#include "cityinfo.h"
#include "army.h"
#include "armysetlist.h"
#include "player.h"
#include "city.h"
#include "GraphicsCache.h"

using namespace std;

CityInfo::CityInfo(City* city)
	:PG_Window(0, PG_Rect((PG_Application::GetScreenWidth()-350)/2,
                (PG_Application::GetScreenHeight()-250)/2, 350, 250)
            , "City info", WF_MODAL), d_city(city)
{
	char buffer[40];
	sprintf(buffer, "Income: %i, Defense:", d_city->getGold());
	new PG_Label(this, PG_Rect(20, 30, my_width - 20, 30), d_city->getName().c_str());
	new PG_Label(this, PG_Rect(20, 60, 150, 20), buffer);

	sprintf(buffer, "%i", d_city->getDefenseLevel());
	d_l_defense = new PG_Label(this, PG_Rect(170, 60, 30, 20), buffer);

	// add an empty button to the buttongroup for "NO PRODUCTION"
	d_b_no_production = new PG_Button(this, 0, PG_Rect(20, 80, 44, 44), "N");
	d_b_no_production->SetEventObject(MSG_BUTTONCLICK, this,
		       (MSG_CALLBACK_OBJ)&CityInfo::b_productionClicked);
	d_b_no_production->SetToggle(true);

	for (int i = 0; i < 4; i++)
	{
		int armytype = d_city->getArmytype(i);
        std::string armyset = d_city->getProductiontype(i).armyset;

		d_b_production[i] = new PG_Button(this, i+1, PG_Rect(64 + i*44, 80, 44, 44));
		d_b_production[i]->SetToggle(true);

		if (armytype != -1) 
        {
            //use GraphicsCache to load army pics because of player-specific
            //colors
            SDL_Surface* armypic = GraphicsCache::getInstance()->getArmyPic(
                            armyset, armytype, d_city->getPlayer());
                            
        	d_b_production[i]->SetIcon(armypic, 0, 0);
			d_b_production[i]->SetEventObject(MSG_BUTTONCLICK, this,
				        (MSG_CALLBACK_OBJ)&CityInfo::b_productionClicked);
		}
		else d_b_production[i]->EnableReceiver(false);
	}

	new PG_Label(this, PG_Rect(20, 130, 80, 20), "Name:");
	new PG_Label(this, PG_Rect(20, 150, 80, 20), "Duration:");
	new PG_Label(this, PG_Rect(20, 170, 80, 20), "Strength:");
	new PG_Label(this, PG_Rect(20, 190, 80, 20), "Moves:");
	new PG_Label(this, PG_Rect(20, 210, 80, 20), "Upkeep:");

	d_l_production = new PG_Label(this, PG_Rect(120, 130, 80, 20), "0");
	d_l_duration = new PG_Label(this, PG_Rect(120, 150, 80, 20), "0");
	d_l_strength = new PG_Label(this, PG_Rect(120, 170, 80, 20), "0");
	d_l_moves = new PG_Label(this, PG_Rect(120, 190, 80, 20), "0");
	d_l_upkeep = new PG_Label(this, PG_Rect(120, 210, 80, 20), "0");

	d_b_upgrade = new PG_Button(this, 1, PG_Rect(my_width - 130, my_height - 110, 110, 30), "Upgrade");
	d_b_buy_production = new PG_Button(this, 2, PG_Rect(my_width - 130, my_height - 75, 110, 30), "Buy production");
	d_b_close = new PG_Button(this, 3, PG_Rect(my_width - 130, my_height - 40, 110, 30), "Close");

    d_b_upgrade->SetEventObject(MSG_BUTTONCLICK, this,
                            (MSG_CALLBACK_OBJ)&CityInfo::b_upgradeClicked);
    d_b_buy_production->SetEventObject(MSG_BUTTONCLICK, this,
				           	(MSG_CALLBACK_OBJ)&CityInfo::b_buy_productionClicked);
	d_b_close->SetEventObject(MSG_BUTTONCLICK, this, (MSG_CALLBACK_OBJ)&CityInfo::b_closeClicked);

	// if something is produced put button to DOWN
	if (d_city->getProduction() != -1) d_b_production[d_city->getProduction()]->SetPressed(true);

	updateProductionStats();
	checkButtons();
}

CityInfo::~CityInfo()
{
	delete d_l_defense;
	delete d_l_production;
	delete d_l_duration;
	delete d_l_strength;
	delete d_l_moves;
	delete d_l_upkeep;
	delete d_b_no_production;
	for (int i = 0; i < 4; i++)
    {
		delete d_b_production[i];
    }
	delete d_b_upgrade;
	delete d_b_buy_production;
	delete d_b_close; 
}

// get the first empty armyproduction-slot of this city

int CityInfo::getEmptySlot()
{
	for (int i = 0; i < 4; i++)
	{
		if (d_city->getArmytype(i) == -1) return i;
	}
	return -1;
}

PARAGUI_CALLBACK(CityInfo::b_productionClicked)
{
	// id is the ID of the button that sent this event
	// id 0 is the "No production" button
	for (int i = 0; i < 4; i++)
	{
		if (i != (id - 1)) d_b_production[i]->SetPressed(false);
	}

	if (id != 0) d_b_no_production->SetPressed(false);
		
	d_city->getPlayer()->cityChangeProduction(d_city, id - 1);
	updateProductionStats();

	return true;
}

void CityInfo::updateProductionStats()
{
	if (d_city->getProduction() >= 0)
	{ 
		// selected armytype
        Armyset_army* a = &Armysetlist::getInstance()->getArmyset(d_city->
                        getProductiontype(d_city->getProduction()).armyset)->
                        army[d_city->getArmytype(d_city->getProduction())];
                            
        d_l_production->SetText(a->getName().c_str());
        char buffer[40];
        sprintf(buffer, "%i", a->getProduction());
        d_l_duration->SetText(buffer);
		sprintf(buffer, "%i", a->getMaxStrength());
		d_l_strength->SetText(buffer);
		sprintf(buffer, "%i", a->getMaxMoves());
		d_l_moves->SetText(buffer);
		sprintf(buffer, "%i", a->getUpkeep());
		d_l_upkeep->SetText(buffer);
	}
	else
	{
		d_l_production->SetText("None");
		d_l_duration->SetText("None");
		d_l_strength->SetText("None");
		d_l_moves->SetText("None");
		d_l_upkeep->SetText("None");
	}
}

PARAGUI_CALLBACK(CityInfo::b_upgradeClicked)
{
	int gold = neededGold();

	if ((gold < 0) || (d_city->getPlayer()->getGold() < gold))
	{
		PG_MessageBox mb(GetParent(), PG_Rect(200, 200, 200, 150),
				"Upgrade City",
				"Not enough gold or already reached level 3!",
				PG_Rect(60, 70, 80, 30), "OK");
		mb.Show();
		mb.WaitForClick();
		mb.Hide();
		return true;
	}

	stringstream s;
	string string;

	s <<"Do you want to upgrade\nthe city to level ";
	s << (d_city->getDefenseLevel() + 1) <<" for " <<gold <<" gold?";
	string = s.str();

	PG_Rect rect_1(20, 110, 80, 30);
	PG_Rect rect_2(105, 110, 80, 30);
	PG_MessageBox mb(GetParent(), PG_Rect(200, 200, 200, 150),
		       "Upgrade city", string.c_str(),
		       rect_1, "Yes", rect_2, "No");
	mb.Show();

	if (mb.WaitForClick() == 1)
	{
		d_city->getPlayer()->cityUpgradeDefense(d_city);

		char buffer[40];
		sprintf(buffer, "%i", d_city->getDefenseLevel());
		d_l_defense->SetText(buffer);
	}
	mb.Hide();

	checkButtons();

	return true;
}

PARAGUI_CALLBACK(CityInfo::b_buy_productionClicked)
{
	D_Buy_Production d_bp(d_city, GetParent(),
                            PG_Rect(my_xpos-50, my_ypos-50, 340, 300));
	d_bp.Show();
	d_bp.RunModal();

	int chosen_army = d_bp.getChosenArmy();

	if (chosen_army == -1)
    {
		return true;
    }
		
	//the only possible error here is a lack of gold, but this
	//should have already been caught in the selection dialogue
	if (!(d_city->getPlayer()->cityBuyProduction(d_city, chosen_army)))
	{
		cerr <<"Player::city_buy_production returned false ";
		cerr <<"where it shouldn't ";
		cerr <<"(CityInfo::buy_production_clicked)\n";
		return true;
	}

    // slot can have values -2, 0, 1, 2
    // -2 comes from "no empty slots"
    // this means that slot 3 was _just_ filled
	int slot = getEmptySlot() - 1;
 	if (slot == -2)
 	{
 		slot = 3;
 	}
	d_b_production[slot]->SetIcon(GraphicsCache::getInstance()->getArmyPic(
                            d_city->getProductiontype(slot).armyset,
                            d_city->getArmytype(slot), d_city->getPlayer()),
                            0, 0);
	d_b_production[slot]->SetEventObject(MSG_BUTTONCLICK, this,
			(MSG_CALLBACK_OBJ)&CityInfo::b_productionClicked);
	d_b_production[slot]->EnableReceiver(true);
	
	checkButtons();
    Redraw();
    
	return true;
}

int CityInfo::neededGold()
{
	if (d_city->getDefenseLevel() == 1) return 1000;
	else if (d_city->getDefenseLevel() == 2) return 2000;
	return -1;
}

void CityInfo::checkButtons()
{
	// Upgrade button
	if ((d_city->getDefenseLevel() < 3 && d_city->getPlayer()->getGold() >= neededGold()))
	{
		d_b_upgrade->EnableReceiver(true);
	}
	else d_b_upgrade->EnableReceiver(false);
	
	// Buy production button
	if (getEmptySlot() != -1) d_b_buy_production->EnableReceiver(true);
	else d_b_buy_production->EnableReceiver(false);
}


PARAGUI_CALLBACK(CityInfo::b_closeClicked)
{
	SendMessage(this, MSG_MODALQUIT, 0, 0);
	return true;
}

// End of file
