//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MAPTILE_H
#define MAPTILE_H

#include "Tile.h"
#include "TileSet.h"

class Maptile
{
	public:
		Maptile(TileSet* tileSet, int x, int y, int type);
		~Maptile();

		enum Building {NONE=0, CITY=1, RUIN=2, TEMPLE=3};

		// Set methods
		void setType(int type){d_type = type;}
		void setCorners(int c1, int c2, int c3, int c4);
		void setMoves(int moves){d_moves = moves;}
		void setBuilding(Building building){d_building = building;}

		// Get methods
		int getType(){return d_type;}
		int getCorner(int c){return d_corner[c];}
		int getBuilding(){return d_building;}
		int getMoves(){return d_moves;}
		int getDefense(){return d_tileSet->getTile(d_type)->getDefense();}
		SDL_Color getColor();
		unsigned int getMaptileType();
        bool isSmoothed(){return d_smoothed;}

		// Methods
		void printDebugInfo();
		float addDefense(float result);
				
	private:
        TileSet* d_tileSet;
		short d_type;		// type (hills, swamp, grass, ...)
		int d_corner[4];    // transition
		short d_moves;		// moves needed to walk over maptile
		Building d_building;// which building is on this maptile
        bool d_smoothed;
};

#endif // MAPTILE_H

// End of file
