//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "player_preferences.h"
#include <list>
#include "File.h"

using namespace std;

Player_preferences::Player_preferences(Type type, string name,
		PG_Widget* parent, PG_Rect rect)
    :PG_Widget(parent, rect)
{   
    d_cb_type = new PG_DropDown(this, 1, PG_Rect(0, 0, 110, 25));
    d_cb_type->AddItem("None");
    d_cb_type->AddItem("Human");
    d_cb_type->AddItem("Computer");

	d_e_name = new PG_LineEdit(this, PG_Rect(120, 0, 100, 25));
	d_e_name->SetText(name.c_str());
    d_cb_armyset = new PG_DropDown(this, 2, PG_Rect(230, 0, 110, 25));

	// fill the armyset combobox with names
	for(list<string>::iterator it = File::s_armySetList.begin(); it != File::s_armySetList.end(); ++it)
	{
		d_cb_armyset->AddItem((*it).c_str());
	}

	d_cb_armyset->SetText("default");
    d_cb_armyset->SetEditable(false);

    d_b_start = new PG_Button(this, 3, PG_Rect(350, 0, 60, 25), "Start");
    d_b_start->SetToggle(true);

	if (type == HUMAN)
	{
        d_cb_type->SetText("Human");
        d_b_start->SetPressed(true);
	}
	else if (type == HUMAN_OR_COMPUTER)
	{
        d_cb_type->SetText("Computer");
        d_b_start->SetPressed(true);
	}
    else
    {
        d_cb_type->SetText("None");
    }

    // set callbacks
    d_b_start->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&Player_preferences::slotPlayerDataChanged);
}

Player_preferences::~Player_preferences()
{
	delete d_cb_type;
	delete d_e_name;
	delete d_cb_armyset;
	delete d_b_start;
}

PARAGUI_CALLBACK(Player_preferences::slotPlayerDataChanged)
{
    playerDataChanged.emit();
    return true;
}

// End of file
